@@ -80,9 +80,9 @@ extern u64 data_bytes_allocated;
 extern u64 data_bytes_referenced;
 extern struct list_head duplicate_extents;
 extern struct list_head delete_items;
-extern int no_holes;
-extern int init_extent_tree;
-extern int check_data_csum;
+extern bool no_holes;
+extern bool init_extent_tree;
+extern bool check_data_csum;
 extern struct btrfs_fs_info *gfs_info;
 extern struct cache_tree *roots_info_cache;
 
@@ -197,5 +197,6 @@ int repair_dev_item_bytes_used(struct btrfs_fs_info *fs_info,
 int fill_csum_tree(struct btrfs_trans_handle *trans, bool search_fs_tree);
 
 int check_and_repair_super_num_devs(struct btrfs_fs_info *fs_info);
+int repair_subvol_orphan_item(struct btrfs_fs_info *fs_info, u64 rootid);
 
 #endif
