@@ -162,8 +162,7 @@ static void print_tree_block_error(struct btrfs_fs_info *fs_info,
 	}
 }
 
-int btrfs_csum_data(struct btrfs_fs_info *fs_info, u16 csum_type, const u8 *data,
-		    u8 *out, size_t len)
+int btrfs_csum_data(u16 csum_type, const u8 *data, u8 *out, size_t len)
 {
 	memset(out, 0, BTRFS_CSUM_SIZE);
 
@@ -191,12 +190,11 @@ static int __csum_tree_block_size(struct extent_buffer *buf, u16 csum_size,
 	u32 len;
 
 	len = buf->len - BTRFS_CSUM_SIZE;
-	btrfs_csum_data(buf->fs_info, csum_type, (u8 *)buf->data + BTRFS_CSUM_SIZE,
-			result, len);
+	btrfs_csum_data(csum_type, (u8 *)buf->data + BTRFS_CSUM_SIZE, result, len);
 
 	if (verify) {
 		if (buf->fs_info && buf->fs_info->skip_csum_check) {
-			/* printf("skip csum check for block %llu\n", buf->start); */
+			/* Skip csum check for block buf->start. */
 		} else if (memcmp_extent_buffer(buf, result, 0, csum_size)) {
 			if (!silent) {
 				char found[BTRFS_CSUM_STRING_LEN];
@@ -817,6 +815,9 @@ struct btrfs_root *btrfs_read_fs_root(struct btrfs_fs_info *fs_info,
 	if (location->objectid == BTRFS_RAID_STRIPE_TREE_OBJECTID)
 		return fs_info->stripe_root ? fs_info->stripe_root : ERR_PTR(-ENOENT);
 
+	if (location->objectid == BTRFS_REMAP_TREE_OBJECTID)
+		return fs_info->remap_root ?: ERR_PTR(-ENOENT);
+
 	BUG_ON(location->objectid == BTRFS_TREE_RELOC_OBJECTID);
 
 	node = rb_search(&fs_info->fs_root_tree, (void *)&objectid,
@@ -852,6 +853,9 @@ void btrfs_free_fs_info(struct btrfs_fs_info *fs_info)
 	if (fs_info->stripe_root)
 		kfree(fs_info->stripe_root);
 
+	if (fs_info->remap_root)
+		kfree(fs_info->remap_root);
+
 	free_global_roots_tree(&fs_info->global_roots_tree);
 	kfree(fs_info->tree_root);
 	kfree(fs_info->chunk_root);
@@ -878,12 +882,13 @@ struct btrfs_fs_info *btrfs_new_fs_info(int writable, u64 sb_bytenr)
 	fs_info->uuid_root = calloc(1, sizeof(struct btrfs_root));
 	fs_info->stripe_root = calloc(1, sizeof(struct btrfs_root));
 	fs_info->block_group_root = calloc(1, sizeof(struct btrfs_root));
+	fs_info->remap_root = calloc(1, sizeof(struct btrfs_root));
 	fs_info->super_copy = calloc(1, BTRFS_SUPER_INFO_SIZE);
 
 	if (!fs_info->tree_root || !fs_info->chunk_root || !fs_info->dev_root ||
 	    !fs_info->quota_root || !fs_info->uuid_root ||
 	    !fs_info->block_group_root || !fs_info->super_copy ||
-	    !fs_info->stripe_root)
+	    !fs_info->stripe_root || !fs_info->remap_root)
 		goto free_all;
 
 	extent_buffer_init_cache(fs_info);
@@ -1228,6 +1233,21 @@ tree_root:
 		return -EIO;
 	}
 
+	if (btrfs_fs_incompat(fs_info, REMAP_TREE) && btrfs_super_remap_root(sb) != 0) {
+		bytenr = btrfs_super_remap_root(sb);
+		gen = btrfs_super_remap_root_generation(sb);
+		level = btrfs_super_remap_root_level(sb);
+
+		root = fs_info->remap_root;
+		btrfs_setup_root(root, fs_info, BTRFS_REMAP_TREE_OBJECTID);
+
+		ret = read_root_node(fs_info, root, bytenr, gen, level);
+		if (ret) {
+			btrfs_warn(fs_info, "couldn't read remap root");
+			return ret;
+		}
+	}
+
 	return 0;
 }
 
@@ -1367,6 +1387,8 @@ void btrfs_release_all_roots(struct btrfs_fs_info *fs_info)
 		free_extent_buffer(fs_info->uuid_root->node);
 	if (fs_info->stripe_root)
 		free_extent_buffer(fs_info->stripe_root->node);
+	if (fs_info->remap_root)
+		free_extent_buffer(fs_info->remap_root->node);
 }
 
 static void free_map_lookup(struct cache_extent *ce)
@@ -1441,6 +1463,7 @@ int btrfs_setup_chunk_tree_and_device_map(struct btrfs_fs_info *fs_info,
 {
 	struct btrfs_super_block *sb = fs_info->super_copy;
 	u64 generation;
+	u8 level;
 	int ret;
 
 	btrfs_setup_root(fs_info->chunk_root, fs_info,
@@ -1450,8 +1473,6 @@ int btrfs_setup_chunk_tree_and_device_map(struct btrfs_fs_info *fs_info,
 	if (ret)
 		return ret;
 
-	generation = btrfs_super_chunk_root_generation(sb);
-
 	if (chunk_root_bytenr && !IS_ALIGNED(chunk_root_bytenr,
 					    fs_info->sectorsize)) {
 		warning("chunk_root_bytenr %llu is unaligned to %u, ignore it",
@@ -1459,13 +1480,28 @@ int btrfs_setup_chunk_tree_and_device_map(struct btrfs_fs_info *fs_info,
 		chunk_root_bytenr = 0;
 	}
 
-	if (!chunk_root_bytenr)
-		chunk_root_bytenr = btrfs_super_chunk_root(sb);
-	else
+	if (chunk_root_bytenr) {
+		struct extent_buffer *eb;
+		struct btrfs_tree_parent_check check = { 0 };
+
 		generation = 0;
+		eb = read_tree_block(fs_info, chunk_root_bytenr, &check);
+		if (IS_ERR(eb)) {
+			ret = PTR_ERR(eb);
+			errno = -ret;
+			error("unable to read tree block at %llu: %m", chunk_root_bytenr);
+			return ret;
+		}
+		level = btrfs_header_level(eb);
+		free_extent_buffer(eb);
+	} else {
+		chunk_root_bytenr = btrfs_super_chunk_root(sb);
+		generation = btrfs_super_chunk_root_generation(sb);
+		level = btrfs_super_chunk_root_level(sb);
+	}
 
 	ret = read_root_node(fs_info, fs_info->chunk_root, chunk_root_bytenr,
-			     generation, btrfs_super_chunk_root_level(sb));
+			     generation, level);
 	if (ret) {
 		if (fs_info->ignore_chunk_tree_error) {
 			warning("cannot read chunk root, continue anyway");
@@ -1508,7 +1544,7 @@ static struct btrfs_fs_info *__open_ctree_fd(int fp, struct open_ctree_args *oca
 
 	fs_info = btrfs_new_fs_info(flags & OPEN_CTREE_WRITES, sb_bytenr);
 	if (!fs_info) {
-		error_msg(ERROR_MSG_MEMORY, "fs_info");
+		error_mem("fs_info");
 		return NULL;
 	}
 	if (flags & OPEN_CTREE_RESTORE)
@@ -1582,17 +1618,16 @@ static struct btrfs_fs_info *__open_ctree_fd(int fp, struct open_ctree_args *oca
 
 	/* CHECK: ignore_csum_mismatch */
 
-	ASSERT(!memcmp(disk_super->fsid, fs_devices->fsid, BTRFS_FSID_SIZE));
+	ASSERT(memcmp(disk_super->fsid, fs_devices->fsid, BTRFS_FSID_SIZE) == 0);
 	if (btrfs_fs_incompat(fs_info, METADATA_UUID))
-		ASSERT(!memcmp(disk_super->metadata_uuid,
-			       fs_devices->metadata_uuid, BTRFS_FSID_SIZE));
+		ASSERT(memcmp(disk_super->metadata_uuid, fs_devices->metadata_uuid,
+			      BTRFS_FSID_SIZE) == 0);
 
 	fs_info->sectorsize = btrfs_super_sectorsize(disk_super);
 	fs_info->nodesize = btrfs_super_nodesize(disk_super);
 	fs_info->stripesize = btrfs_super_stripesize(disk_super);
 	fs_info->csum_type = btrfs_super_csum_type(disk_super);
 	fs_info->csum_size = btrfs_super_csum_size(disk_super);
-	fs_info->leaf_data_size = __BTRFS_LEAF_DATA_SIZE(fs_info->nodesize);
 
 	ret = btrfs_check_fs_compatibility(fs_info->super_copy, flags);
 	if (ret)
@@ -1639,6 +1674,11 @@ static struct btrfs_fs_info *__open_ctree_fd(int fp, struct open_ctree_args *oca
 		goto out_chunk;
 	}
 
+	if (btrfs_fs_incompat(fs_info, REMAP_TREE)) {
+		btrfs_setup_root(fs_info->remap_root, fs_info,
+				 BTRFS_REMAP_TREE_OBJECTID);
+	}
+
 	eb = fs_info->chunk_root->node;
 	read_extent_buffer(eb, fs_info->chunk_tree_uuid,
 			   btrfs_header_chunk_tree_uuid(eb),
@@ -1770,11 +1810,17 @@ int btrfs_check_super(struct btrfs_super_block *sb, unsigned sbflags)
 	}
 	csum_size = btrfs_super_csum_size(sb);
 
-	btrfs_csum_data(NULL, csum_type, (u8 *)sb + BTRFS_CSUM_SIZE,
+	btrfs_csum_data(csum_type, (u8 *)sb + BTRFS_CSUM_SIZE,
 			result, BTRFS_SUPER_INFO_SIZE - BTRFS_CSUM_SIZE);
 
-	if (memcmp(result, sb->csum, csum_size)) {
-		error("superblock checksum mismatch");
+	if (memcmp(result, sb->csum, csum_size) != 0) {
+		char found[BTRFS_CSUM_STRING_LEN];
+		char wanted[BTRFS_CSUM_STRING_LEN];
+
+		btrfs_format_csum(csum_type, result, found);
+		btrfs_format_csum(csum_type, (u8 *)sb->csum, wanted);
+
+		error("superblock checksum mismatch: wanted %s found %s", wanted, found);
 		return -EIO;
 	}
 	if (btrfs_super_root_level(sb) >= BTRFS_MAX_LEVEL) {
@@ -1816,13 +1862,10 @@ int btrfs_check_super(struct btrfs_super_block *sb, unsigned sbflags)
 		error("nodesize unaligned: %u", btrfs_super_nodesize(sb));
 		goto error_out;
 	}
-	if (btrfs_super_sectorsize(sb) < 4096) {
-		error("sectorsize too small: %u < 4096",
-			btrfs_super_sectorsize(sb));
-		goto error_out;
-	}
-	if (!IS_ALIGNED(btrfs_super_sectorsize(sb), 4096)) {
-		error("sectorsize unaligned: %u", btrfs_super_sectorsize(sb));
+	if (!is_power_of_2(btrfs_super_sectorsize(sb)) ||
+	    btrfs_super_sectorsize(sb) < BTRFS_MIN_BLOCKSIZE ||
+	    btrfs_super_sectorsize(sb) > BTRFS_MAX_METADATA_BLOCKSIZE) {
+		error("invalid sectorsize: %u", btrfs_super_sectorsize(sb));
 		goto error_out;
 	}
 	if (btrfs_super_total_bytes(sb) == 0) {
@@ -2031,7 +2074,7 @@ static int write_dev_supers(struct btrfs_fs_info *fs_info,
 	}
 	if (fs_info->super_bytenr != BTRFS_SUPER_INFO_OFFSET) {
 		btrfs_set_super_bytenr(sb, fs_info->super_bytenr);
-		btrfs_csum_data(fs_info, csum_type, (u8 *)sb + BTRFS_CSUM_SIZE,
+		btrfs_csum_data(csum_type, (u8 *)sb + BTRFS_CSUM_SIZE,
 				result, BTRFS_SUPER_INFO_SIZE - BTRFS_CSUM_SIZE);
 		memcpy(&sb->csum[0], result, BTRFS_CSUM_SIZE);
 
@@ -2064,7 +2107,7 @@ static int write_dev_supers(struct btrfs_fs_info *fs_info,
 
 		btrfs_set_super_bytenr(sb, bytenr);
 
-		btrfs_csum_data(fs_info, csum_type, (u8 *)sb + BTRFS_CSUM_SIZE,
+		btrfs_csum_data(csum_type, (u8 *)sb + BTRFS_CSUM_SIZE,
 				result, BTRFS_SUPER_INFO_SIZE - BTRFS_CSUM_SIZE);
 		memcpy(&sb->csum[0], result, BTRFS_CSUM_SIZE);
 
@@ -2256,6 +2299,11 @@ int close_ctree_fs_info(struct btrfs_fs_info *fs_info)
 	struct btrfs_trans_handle *trans;
 	struct btrfs_root *root = fs_info->tree_root;
 
+	if (fs_info->transaction_aborted && fs_info->running_transaction) {
+		btrfs_cleanup_aborted_transaction(fs_info);
+		goto skip_commit;
+	}
+
 	if (fs_info->last_trans_committed !=
 	    fs_info->generation) {
 		BUG_ON(!root);
