@@ -15,7 +15,6 @@
 # You should have received a copy of the GNU Lesser General Public License
 # along with libbtrfsutil.  If not, see <http://www.gnu.org/licenses/>.
 
-import fcntl
 import errno
 import multiprocessing
 import os
@@ -54,6 +53,25 @@ class TestSubvolume(BtrfsTestCase):
         self.assertEqual(e.exception.btrfsutilerror, btrfsutil.ERROR_STATFS_FAILED)
         self.assertEqual(e.exception.errno, errno.ENOENT)
 
+    # Copy of test_is_subvolume
+    def test_subvolume_is_valid(self):
+        dir = os.path.join(self.mountpoint, 'foo')
+        os.mkdir(dir)
+
+        for arg in self.path_or_fd(self.mountpoint):
+            with self.subTest(type=type(arg)):
+                self.assertTrue(btrfsutil.subvolume_is_valid(arg))
+        for arg in self.path_or_fd(dir):
+            with self.subTest(type=type(arg)):
+                self.assertFalse(btrfsutil.subvolume_is_valid(arg))
+
+        with self.assertRaises(btrfsutil.BtrfsUtilError) as e:
+            btrfsutil.subvolume_is_valid(os.path.join(self.mountpoint, 'bar'))
+        # This is a bit of an implementation detail, but really this is testing
+        # that the exception is initialized correctly.
+        self.assertEqual(e.exception.btrfsutilerror, btrfsutil.ERROR_STATFS_FAILED)
+        self.assertEqual(e.exception.errno, errno.ENOENT)
+
     def test_subvolume_id(self):
         dir = os.path.join(self.mountpoint, 'foo')
         os.mkdir(dir)
@@ -65,10 +83,22 @@ class TestSubvolume(BtrfsTestCase):
             with self.subTest(type=type(arg)):
                 self.assertEqual(btrfsutil.subvolume_id(arg), 5)
 
+    # Copy of test_subvolume_id
+    def test_subvolume_get_id(self):
+        dir = os.path.join(self.mountpoint, 'foo')
+        os.mkdir(dir)
+
+        for arg in self.path_or_fd(self.mountpoint):
+            with self.subTest(type=type(arg)):
+                self.assertEqual(btrfsutil.subvolume_get_id(arg), 5)
+        for arg in self.path_or_fd(dir):
+            with self.subTest(type=type(arg)):
+                self.assertEqual(btrfsutil.subvolume_get_id(arg), 5)
+
     def test_subvolume_id_error(self):
         fd = os.open('/dev/null', os.O_RDONLY)
         try:
-            btrfsutil.subvolume_id(fd)
+            btrfsutil.subvolume_get_id(fd)
         except Exception:
             pass
         finally:
@@ -108,6 +138,38 @@ class TestSubvolume(BtrfsTestCase):
         finally:
             os.chdir(pwd)
 
+    # Copy of test_subvolume_path
+    def test_subvolume_get_path(self):
+        btrfsutil.subvolume_create(os.path.join(self.mountpoint, 'subvol1'))
+        os.mkdir(os.path.join(self.mountpoint, 'dir1'))
+        os.mkdir(os.path.join(self.mountpoint, 'dir1/dir2'))
+        btrfsutil.subvolume_create(os.path.join(self.mountpoint, 'dir1/dir2/subvol2'))
+        btrfsutil.subvolume_create(os.path.join(self.mountpoint, 'dir1/dir2/subvol2/subvol3'))
+        os.mkdir(os.path.join(self.mountpoint, 'subvol1/dir3'))
+        btrfsutil.subvolume_create(os.path.join(self.mountpoint, 'subvol1/dir3/subvol4'))
+
+        for arg in self.path_or_fd(self.mountpoint):
+            with self.subTest(type=type(arg)):
+                self.assertEqual(btrfsutil.subvolume_get_path(arg), '')
+                self.assertEqual(btrfsutil.subvolume_get_path(arg, 5), '')
+                self.assertEqual(btrfsutil.subvolume_get_path(arg, 256), 'subvol1')
+                self.assertEqual(btrfsutil.subvolume_get_path(arg, 257), 'dir1/dir2/subvol2')
+                self.assertEqual(btrfsutil.subvolume_get_path(arg, 258), 'dir1/dir2/subvol2/subvol3')
+                self.assertEqual(btrfsutil.subvolume_get_path(arg, 259), 'subvol1/dir3/subvol4')
+
+        pwd = os.getcwd()
+        try:
+            os.chdir(self.mountpoint)
+            path = ''
+            for i in range(26):
+                name = chr(ord('a') + i) * 255
+                path = os.path.join(path, name)
+                btrfsutil.subvolume_create(name)
+                os.chdir(name)
+            self.assertEqual(btrfsutil.subvolume_get_path('.'), path)
+        finally:
+            os.chdir(pwd)
+
     def _test_subvolume_info(self, subvol, snapshot):
         for arg in self.path_or_fd(self.mountpoint):
             with self.subTest(type=type(arg)):
@@ -156,6 +218,55 @@ class TestSubvolume(BtrfsTestCase):
 
         # TODO: test received_uuid, stransid, rtransid, stime, and rtime
 
+    # Copy of _test_subvolume_info
+    def _test_subvolume_get_info(self, subvol, snapshot):
+        for arg in self.path_or_fd(self.mountpoint):
+            with self.subTest(type=type(arg)):
+                info = btrfsutil.subvolume_get_info(arg)
+                self.assertEqual(info.id, 5)
+                self.assertEqual(info.parent_id, 0)
+                self.assertEqual(info.dir_id, 0)
+                self.assertEqual(info.flags, 0)
+                self.assertIsInstance(info.uuid, bytes)
+                self.assertEqual(len(info.uuid), 16)
+                self.assertEqual(info.parent_uuid, bytes(16))
+                self.assertEqual(info.received_uuid, bytes(16))
+                self.assertNotEqual(info.generation, 0)
+                self.assertGreaterEqual(info.ctransid, 0)
+                self.assertEqual(info.otransid, 0)
+                self.assertEqual(info.stransid, 0)
+                self.assertEqual(info.rtransid, 0)
+                self.assertIsInstance(info.ctime, float)
+                self.assertIsInstance(info.otime, float)
+                self.assertEqual(info.stime, 0)
+                self.assertEqual(info.rtime, 0)
+
+        info = btrfsutil.subvolume_get_info(subvol)
+        self.assertEqual(info.id, 256)
+        self.assertEqual(info.parent_id, 5)
+        self.assertEqual(info.dir_id, 256)
+        self.assertEqual(info.flags, 0)
+        self.assertIsInstance(info.uuid, bytes)
+        self.assertEqual(len(info.uuid), 16)
+        self.assertEqual(info.parent_uuid, bytes(16))
+        self.assertEqual(info.received_uuid, bytes(16))
+        self.assertNotEqual(info.generation, 0)
+        self.assertNotEqual(info.ctransid, 0)
+        self.assertNotEqual(info.otransid, 0)
+        self.assertEqual(info.stransid, 0)
+        self.assertEqual(info.rtransid, 0)
+        self.assertNotEqual(info.ctime, 0)
+        self.assertNotEqual(info.otime, 0)
+        self.assertEqual(info.stime, 0)
+        self.assertEqual(info.rtime, 0)
+
+        subvol_uuid = info.uuid
+
+        info = btrfsutil.subvolume_get_info(snapshot)
+        self.assertEqual(info.parent_uuid, subvol_uuid)
+
+        # TODO: test received_uuid, stransid, rtransid, stime, and rtime
+
     def test_subvolume_info(self):
         subvol = os.path.join(self.mountpoint, 'subvol')
         btrfsutil.create_subvolume(subvol)
@@ -172,6 +283,23 @@ class TestSubvolume(BtrfsTestCase):
                 self.assertEqual(e.exception.btrfsutilerror,
                                  btrfsutil.ERROR_SUBVOLUME_NOT_FOUND)
 
+    # Copy of test_subvolume_info
+    def test_subvolume_get_info(self):
+        subvol = os.path.join(self.mountpoint, 'subvol')
+        btrfsutil.subvolume_create(subvol)
+        snapshot = os.path.join(self.mountpoint, 'snapshot')
+        btrfsutil.subvolume_snapshot(subvol, snapshot)
+
+        self._test_subvolume_info(subvol, snapshot)
+
+        for arg in self.path_or_fd(self.mountpoint):
+            with self.subTest(type=type(arg)):
+                with self.assertRaises(btrfsutil.BtrfsUtilError) as e:
+                    # BTRFS_EXTENT_TREE_OBJECTID
+                    btrfsutil.subvolume_get_info(arg, 2)
+                self.assertEqual(e.exception.btrfsutilerror,
+                                 btrfsutil.ERROR_SUBVOLUME_NOT_FOUND)
+
     @skipUnlessHaveNobody
     def test_subvolume_info_unprivileged(self):
         subvol = os.path.join(self.mountpoint, 'subvol')
@@ -189,6 +317,24 @@ class TestSubvolume(BtrfsTestCase):
                     raise
             self._test_subvolume_info(subvol, snapshot)
 
+    # Copy of test_subvolume_info_unprivileged
+    @skipUnlessHaveNobody
+    def test_subvolume_get_info_unprivileged(self):
+        subvol = os.path.join(self.mountpoint, 'subvol')
+        btrfsutil.subvolume_create(subvol)
+        snapshot = os.path.join(self.mountpoint, 'snapshot')
+        btrfsutil.subvolume_snapshot(subvol, snapshot)
+
+        with drop_privs():
+            try:
+                btrfsutil.subvolume_get_info(self.mountpoint)
+            except OSError as e:
+                if e.errno == errno.ENOTTY:
+                    self.skipTest('BTRFS_IOC_GET_SUBVOL_INFO is not available')
+                else:
+                    raise
+            self._test_subvolume_info(subvol, snapshot)
+
     def test_read_only(self):
         for arg in self.path_or_fd(self.mountpoint):
             with self.subTest(type=type(arg)):
@@ -206,6 +352,24 @@ class TestSubvolume(BtrfsTestCase):
 
                 btrfsutil.set_subvolume_read_only(arg, False)
 
+    # Copy of test_read_only
+    def test_subvolume_read_only(self):
+        for arg in self.path_or_fd(self.mountpoint):
+            with self.subTest(type=type(arg)):
+                btrfsutil.subvolume_set_read_only(arg)
+                self.assertTrue(btrfsutil.subvolume_get_read_only(arg))
+                self.assertTrue(btrfsutil.subvolume_get_info(arg).flags & 1)
+
+                btrfsutil.subvolume_set_read_only(arg, False)
+                self.assertFalse(btrfsutil.subvolume_get_read_only(arg))
+                self.assertFalse(btrfsutil.subvolume_get_info(arg).flags & 1)
+
+                btrfsutil.subvolume_set_read_only(arg, True)
+                self.assertTrue(btrfsutil.subvolume_get_read_only(arg))
+                self.assertTrue(btrfsutil.subvolume_get_info(arg).flags & 1)
+
+                btrfsutil.subvolume_set_read_only(arg, False)
+
     def test_default_subvolume(self):
         for arg in self.path_or_fd(self.mountpoint):
             with self.subTest(type=type(arg)):
@@ -220,6 +384,21 @@ class TestSubvolume(BtrfsTestCase):
                 btrfsutil.set_default_subvolume(arg, 5)
                 self.assertEqual(btrfsutil.get_default_subvolume(arg), 5)
 
+    # Copy of test_default_subvolume
+    def test_subvolume_default(self):
+        for arg in self.path_or_fd(self.mountpoint):
+            with self.subTest(type=type(arg)):
+                self.assertEqual(btrfsutil.subvolume_get_default(arg), 5)
+
+        subvol = os.path.join(self.mountpoint, 'subvol')
+        btrfsutil.subvolume_create(subvol)
+        for arg in self.path_or_fd(subvol):
+            with self.subTest(type=type(arg)):
+                btrfsutil.subvolume_set_default(arg)
+                self.assertEqual(btrfsutil.subvolume_get_default(arg), 256)
+                btrfsutil.subvolume_set_default(arg, 5)
+                self.assertEqual(btrfsutil.subvolume_get_default(arg), 5)
+
     def test_create_subvolume(self):
         subvol = os.path.join(self.mountpoint, 'subvol')
 
@@ -263,6 +442,50 @@ class TestSubvolume(BtrfsTestCase):
         self.assertTrue(os.WIFEXITED(wstatus))
         self.assertEqual(os.WEXITSTATUS(wstatus), 0)
 
+    # Copy of test_create_subvolume
+    def test_subvolume_create(self):
+        subvol = os.path.join(self.mountpoint, 'subvol')
+
+        btrfsutil.subvolume_create(subvol + '1')
+        self.assertTrue(btrfsutil.subvolume_is_valid(subvol + '1'))
+        btrfsutil.subvolume_create((subvol + '2').encode())
+        self.assertTrue(btrfsutil.subvolume_is_valid(subvol + '2'))
+        if HAVE_PATH_LIKE:
+            btrfsutil.subvolume_create(PurePath(subvol + '3'))
+            self.assertTrue(btrfsutil.subvolume_is_valid(subvol + '3'))
+
+        pwd = os.getcwd()
+        try:
+            os.chdir(self.mountpoint)
+            btrfsutil.subvolume_create('subvol4')
+            self.assertTrue(btrfsutil.subvolume_is_valid('subvol4'))
+        finally:
+            os.chdir(pwd)
+
+        btrfsutil.subvolume_create(subvol + '5/')
+        self.assertTrue(btrfsutil.subvolume_is_valid(subvol + '5'))
+
+        btrfsutil.subvolume_create(subvol + '6//')
+        self.assertTrue(btrfsutil.subvolume_is_valid(subvol + '6'))
+
+        # Test creating subvolumes under '/' in a chroot.
+        pid = os.fork()
+        if pid == 0:
+            try:
+                os.chroot(self.mountpoint)
+                os.chdir('/')
+                btrfsutil.subvolume_create('/subvol8')
+                self.assertTrue(btrfsutil.subvolume_is_valid('/subvol8'))
+                with self.assertRaises(btrfsutil.BtrfsUtilError):
+                    btrfsutil.subvolume_create('/')
+                os._exit(0)
+            except Exception:
+                traceback.print_exc()
+                os._exit(1)
+        wstatus = os.waitpid(pid, 0)[1]
+        self.assertTrue(os.WIFEXITED(wstatus))
+        self.assertEqual(os.WEXITSTATUS(wstatus), 0)
+
     def test_create_snapshot(self):
         subvol = os.path.join(self.mountpoint, 'subvol')
 
@@ -307,6 +530,51 @@ class TestSubvolume(BtrfsTestCase):
         btrfsutil.create_snapshot(subvol, snapshot + '3', read_only=True)
         self.assertTrue(btrfsutil.get_subvolume_read_only(snapshot + '3'))
 
+    # Copy of test_create_snapshot
+    def test_subvolume_snapshot(self):
+        subvol = os.path.join(self.mountpoint, 'subvol')
+
+        btrfsutil.subvolume_create(subvol)
+        os.mkdir(os.path.join(subvol, 'dir'))
+
+        for i, arg in enumerate(self.path_or_fd(subvol)):
+            with self.subTest(type=type(arg)):
+                snapshots_dir = os.path.join(self.mountpoint, 'snapshots{}'.format(i))
+                os.mkdir(snapshots_dir)
+                snapshot = os.path.join(snapshots_dir, 'snapshot')
+
+                btrfsutil.subvolume_snapshot(subvol, snapshot + '1')
+                self.assertTrue(btrfsutil.subvolume_is_valid(snapshot + '1'))
+                self.assertTrue(os.path.exists(os.path.join(snapshot + '1', 'dir')))
+
+                btrfsutil.subvolume_snapshot(subvol, (snapshot + '2').encode())
+                self.assertTrue(btrfsutil.subvolume_is_valid(snapshot + '2'))
+                self.assertTrue(os.path.exists(os.path.join(snapshot + '2', 'dir')))
+
+                if HAVE_PATH_LIKE:
+                    btrfsutil.subvolume_snapshot(subvol, PurePath(snapshot + '3'))
+                    self.assertTrue(btrfsutil.subvolume_is_valid(snapshot + '3'))
+                    self.assertTrue(os.path.exists(os.path.join(snapshot + '3', 'dir')))
+
+        nested_subvol = os.path.join(subvol, 'nested')
+        more_nested_subvol = os.path.join(nested_subvol, 'more_nested')
+        btrfsutil.subvolume_create(nested_subvol)
+        btrfsutil.subvolume_create(more_nested_subvol)
+        os.mkdir(os.path.join(more_nested_subvol, 'nested_dir'))
+
+        snapshot = os.path.join(self.mountpoint, 'snapshot')
+
+        btrfsutil.subvolume_snapshot(subvol, snapshot + '1')
+        # Dummy subvolume.
+        self.assertEqual(os.stat(os.path.join(snapshot + '1', 'nested')).st_ino, 2)
+        self.assertFalse(os.path.exists(os.path.join(snapshot + '1', 'nested', 'more_nested')))
+
+        btrfsutil.subvolume_snapshot(subvol, snapshot + '2', recursive=True)
+        self.assertTrue(os.path.exists(os.path.join(snapshot + '2', 'nested/more_nested/nested_dir')))
+
+        btrfsutil.subvolume_snapshot(subvol, snapshot + '3', read_only=True)
+        self.assertTrue(btrfsutil.subvolume_get_read_only(snapshot + '3'))
+
     def test_delete_subvolume(self):
         subvol = os.path.join(self.mountpoint, 'subvol')
         btrfsutil.create_subvolume(subvol + '1')
@@ -355,6 +623,55 @@ class TestSubvolume(BtrfsTestCase):
         btrfsutil.delete_subvolume(subvol + '5', recursive=True)
         self.assertFalse(os.path.exists(subvol + '5'))
 
+    # Copy of test_delete_subvolume
+    def test_subvolume_delete(self):
+        subvol = os.path.join(self.mountpoint, 'subvol')
+        btrfsutil.subvolume_create(subvol + '1')
+        self.assertTrue(os.path.exists(subvol + '1'))
+        btrfsutil.subvolume_create(subvol + '2')
+        self.assertTrue(os.path.exists(subvol + '2'))
+        btrfsutil.subvolume_create(subvol + '3')
+        self.assertTrue(os.path.exists(subvol + '3'))
+
+        btrfsutil.subvolume_delete(subvol + '1')
+        self.assertFalse(os.path.exists(subvol + '1'))
+        btrfsutil.subvolume_delete((subvol + '2').encode())
+        self.assertFalse(os.path.exists(subvol + '2'))
+        if HAVE_PATH_LIKE:
+            btrfsutil.subvolume_delete(PurePath(subvol + '3'))
+            self.assertFalse(os.path.exists(subvol + '3'))
+
+        # Test deleting subvolumes under '/' in a chroot.
+        pid = os.fork()
+        if pid == 0:
+            try:
+                os.chroot(self.mountpoint)
+                os.chdir('/')
+                btrfsutil.subvolume_create('/subvol4')
+                self.assertTrue(os.path.exists('/subvol4'))
+                btrfsutil.subvolume_delete('/subvol4')
+                self.assertFalse(os.path.exists('/subvol4'))
+                with self.assertRaises(btrfsutil.BtrfsUtilError):
+                    btrfsutil.subvolume_delete('/')
+                os._exit(0)
+            except Exception:
+                traceback.print_exc()
+                os._exit(1)
+        wstatus = os.waitpid(pid, 0)[1]
+        self.assertTrue(os.WIFEXITED(wstatus))
+        self.assertEqual(os.WEXITSTATUS(wstatus), 0)
+
+        btrfsutil.subvolume_create(subvol + '5')
+        btrfsutil.subvolume_create(subvol + '5/foo')
+        btrfsutil.subvolume_create(subvol + '5/bar')
+        btrfsutil.subvolume_create(subvol + '5/bar/baz')
+        btrfsutil.subvolume_create(subvol + '5/bar/qux')
+        btrfsutil.subvolume_create(subvol + '5/quux')
+        with self.assertRaises(btrfsutil.BtrfsUtilError):
+            btrfsutil.subvolume_delete(subvol + '5')
+        btrfsutil.subvolume_delete(subvol + '5', recursive=True)
+        self.assertFalse(os.path.exists(subvol + '5'))
+
     def test_deleted_subvolumes(self):
         subvol = os.path.join(self.mountpoint, 'subvol')
         btrfsutil.create_subvolume(subvol + '1')
@@ -363,8 +680,18 @@ class TestSubvolume(BtrfsTestCase):
             with self.subTest(type=type(arg)):
                 self.assertEqual(btrfsutil.deleted_subvolumes(arg), [256])
 
+    # Copy of test_deleted_subvolumes
+    def test_subvolume_list_deleted(self):
+        subvol = os.path.join(self.mountpoint, 'subvol')
+        btrfsutil.subvolume_create(subvol + '1')
+        btrfsutil.subvolume_delete(subvol + '1')
+        for arg in self.path_or_fd(self.mountpoint):
+            with self.subTest(type=type(arg)):
+                self.assertEqual(btrfsutil.subvolume_list_deleted(arg), [256])
+
+    # Only the new API
     def _test_subvolume_iterator(self):
-        btrfsutil.create_subvolume('foo')
+        btrfsutil.subvolume_create('foo')
 
         with btrfsutil.SubvolumeIterator('.', info=True) as it:
             path, subvol = next(it)
@@ -374,8 +701,8 @@ class TestSubvolume(BtrfsTestCase):
             self.assertEqual(subvol.parent_id, 5)
             self.assertRaises(StopIteration, next, it)
 
-        btrfsutil.create_subvolume('foo/bar')
-        btrfsutil.create_subvolume('foo/bar/baz')
+        btrfsutil.subvolume_create('foo/bar')
+        btrfsutil.subvolume_create('foo/bar/baz')
 
         subvols = [
             ('foo', 256),
@@ -413,9 +740,9 @@ class TestSubvolume(BtrfsTestCase):
 
         os.rename('foo/bar/baz', 'baz')
         os.mkdir('dir')
-        btrfsutil.create_subvolume('dir/qux')
+        btrfsutil.subvolume_create('dir/qux')
         os.mkdir('dir/qux/dir2')
-        btrfsutil.create_subvolume('dir/qux/dir2/quux')
+        btrfsutil.subvolume_create('dir/qux/dir2/quux')
 
         subvols = [
             ('baz', 258),
@@ -431,28 +758,28 @@ class TestSubvolume(BtrfsTestCase):
             with regain_privs():
                 # We don't have permission to traverse the path.
                 os.mkdir('directory_perms', 0o700)
-                btrfsutil.create_subvolume('directory_perms/subvol')
+                btrfsutil.subvolume_create('directory_perms/subvol')
 
                 # We don't have permission to resolve the subvolume path.
                 os.mkdir('subvol_perms', 0o755)
-                btrfsutil.create_subvolume('subvol_perms/subvol')
+                btrfsutil.subvolume_create('subvol_perms/subvol')
                 os.chmod('subvol_perms/subvol', 0o700)
 
                 # The path doesn't exist.
                 os.mkdir('enoent', 0o755)
-                btrfsutil.create_subvolume('enoent/subvol')
+                btrfsutil.subvolume_create('enoent/subvol')
                 subprocess.check_call(['mount', '-t', 'tmpfs', 'tmpfs', 'enoent'])
 
                 # The path exists but it's not a subvolume.
                 os.mkdir('notsubvol', 0o755)
-                btrfsutil.create_subvolume('notsubvol/subvol')
+                btrfsutil.subvolume_create('notsubvol/subvol')
                 subprocess.check_call(['mount', '-t', 'tmpfs', 'tmpfs', 'notsubvol'])
                 os.mkdir('notsubvol/subvol')
 
                 # The path exists and is a subvolume, but on a different
                 # filesystem.
                 os.mkdir('wrongfs', 0o755)
-                btrfsutil.create_subvolume('wrongfs/subvol')
+                btrfsutil.subvolume_create('wrongfs/subvol')
                 other_mountpoint, _ = self.mount_btrfs()
                 subprocess.check_call(['mount', '--bind', '--',
                                        other_mountpoint, 'wrongfs/subvol'])
@@ -460,7 +787,7 @@ class TestSubvolume(BtrfsTestCase):
                 # The path exists and is a subvolume on the same
                 # filesystem, but not the right one.
                 os.mkdir('wrongsubvol', 0o755)
-                btrfsutil.create_subvolume('wrongsubvol/subvol')
+                btrfsutil.subvolume_create('wrongsubvol/subvol')
                 subprocess.check_call(['mount', '--bind', 'baz', 'wrongsubvol/subvol'])
 
 
@@ -509,16 +836,18 @@ class TestSubvolume(BtrfsTestCase):
         finally:
             os.chdir(pwd)
 
+    # Only the new API
     @staticmethod
     def _create_and_delete_subvolume(i):
         dir_name = f'dir{i}'
         subvol_name = dir_name + '/subvol'
         while True:
             os.mkdir(dir_name)
-            btrfsutil.create_subvolume(subvol_name)
-            btrfsutil.delete_subvolume(subvol_name)
+            btrfsutil.subvolume_create(subvol_name)
+            btrfsutil.subvolume_delete(subvol_name)
             os.rmdir(dir_name)
 
+    # Only the new API
     def _test_subvolume_iterator_race(self):
         procs = []
         fd = os.open('.', os.O_RDONLY | os.O_DIRECTORY)
@@ -543,6 +872,7 @@ class TestSubvolume(BtrfsTestCase):
                 proc.join()
             os.close(fd)
 
+    # Only the new API
     def test_subvolume_iterator_race(self):
         pwd = os.getcwd()
         try:
@@ -551,6 +881,7 @@ class TestSubvolume(BtrfsTestCase):
         finally:
             os.chdir(pwd)
 
+    # Only the new API
     def test_subvolume_iterator_race_unprivileged(self):
         os.chown(self.mountpoint, NOBODY_UID, -1)
         pwd = os.getcwd()
@@ -562,11 +893,12 @@ class TestSubvolume(BtrfsTestCase):
         finally:
             os.chdir(pwd)
 
+    # Only the new API
     def test_subvolume_iterator_fd_unprivileged(self):
         pwd = os.getcwd()
         try:
             os.chdir(self.mountpoint)
-            btrfsutil.create_subvolume('subvol')
+            btrfsutil.subvolume_create('subvol')
             with drop_privs():
                 fd = os.open('.', os.O_RDONLY | os.O_DIRECTORY)
                 try:
